C==============================================================================
C-----Fortran subroutines to read and write MatLab .mat files on format MatLab 4.2
C-----Higher versions of MatLab can read .mat file at this format
C-----Higher versions of MatLab can write .mat file at this format using -V4 option
C-----Ex: "save file.mat value1 value2 -V4"
C-----Pierre-Olivier Malaterre  - 3 feb 2000
C==============================================================================

      SUBROUTINE MKM2( A, Name, namlen, N, NI, NJ, IOflag )
C-----A Fortran subroutine that writes a real double-precision matrix
C-----to a data file "slicotou.mat" in the .MAT-file format used by MATLAB 4.2b.
C-----A: matrix (double precision)
C-----Name: name of the matrix
C-----namlen: length of the Name in number of characters
C-----(this is required since there is no way to know this length in this subroutine)
C-----N: row dimension of A (max)
C-----NI: number of rows in A (active)
C-----NJ: number of columns in A (active)
C-----IOFlag = 1 for the first write (and open the slicotou.mat file)
C-----IOFlag = 0 for the next write
C-----IOFlag = -1 for the last write (and close the slicotou.mat file)

      DOUBLE PRECISION A(N,*)
      DOUBLE PRECISION B
      character  Name*(*)
      integer namlen, N, NI, NJ, IOFlag
      integer*1  zerobyte
      data zerobyte / 0 /

      IF(IOFlag.EQ.1) THEN
        ILU1=80
        open(unit=ILU1,file='slicotou.mat',status='unknown'
     &      ,form='binary')
      ENDIF

c
c  write header
      write(ILU1,err=999)  0,NI,NJ,0,namlen+1
c
c  look for the beginning of variable Name (to remove initial spaces if any)
c  in fact I do not know if this task is required. It seems that if the
c  Name is provided through a fixed value this is not possible, but if it is
c  provided through a character*n variable, then an initial space is added or
c  the Name can be formated on the right side, and these spaces should be removed

      DO 5 I=1,namlen
        IF(Name(I:I).NE.' ') THEN
          I0=I
          I1=I0+namlen-1
          GOTO 6
        ENDIF
    5 CONTINUE
    6 CONTINUE

c  write variable name
      write(ILU1,err=999) (Name(i:i),i=I0,I1),zerobyte
c
c  write real part of variable
      DO 10 J=1,NJ
        DO 20 I=1,NI
          B = A(I,J)
          write(ILU1,err=999)  B
   20   CONTINUE
   10 CONTINUE

      IF(IOFLAG.EQ.-1) THEN
        close(ILU1)
      ENDIF
c
  999 continue
      end



      SUBROUTINE RDM2(A,N,M,mrows,ncols,IOFlag,rdflg)
c     A Fortran subroutine that reads a real double-precision matrix from a
c     data file (slicotin.mat) in the .MAT-file format used by MATLAB 4.2b.
c     Description of inputs:
C       N max number of rows in A
C       M max number of cols in A
c       IOFlag  : 1 Open the file slicotin.mat; 0 Normal; -1 Close the file
c     Description of outputs:
c       A       - matrix
c       type    - matrix type flag
c       mrows   - number of rows in matrix
c       ncols   - number of columns in matrix
c       imagf   - imaginary flag; 0 for no imaginary part or 1 if the
c                 data has an imaginary part
c       namlen  - number of characters in matrix name plus 1 (for zero
c                 byte string terminator)
c       name    - character array holding the matrix name. (Be sure
c                 character array has room for the zero byte terminator)
c       rpart   - real part of matrix (mrows x ncols double precision
c                 elements stored column wise)
c       ipart   - imaginary part of matrix (only used if imagf = 1)
c       rdflg   - read status flag; = 0 good read, = 1 end of file,
c                 = -1 error reading file, = -2 dim problem
c
c       o  Each call to this routine will read another MATLAB
c          matrix until an error occurs or an end-of-file is reached.
c       o  Be sure that the 4 byte integer option is used when compiling
c          this routine and that LOADMAT's integer arguments are 4 byte.
c
      DOUBLE PRECISION A(N,*)
      integer  type, mrows, ncols, imagf, namlen
C-----The name must be less than 9 characters
      character  name(10)*1
      double precision  rpart
      integer  rdflg
      integer*1 zerobyte
c
      IF(IOFlag.EQ.1) THEN
        ILU2=90
        open(unit=ILU2,file='slicotin.mat',status='old',form='binary')
      ENDIF
c
c  read header
      read(ILU2,err=998,end=999) type,mrows,ncols,imagf,namlen
C-----management of problems on dimensions (POM 4/10/99)
C-----in this case only the possible sub matrix will be read
      if (mrows.GT.N) then
         WRITE(*,*) 'Error: matrix too large in .mat file (rows)'
      endif
      if (ncols.GT.M) then
         WRITE(*,*) 'Error: matrix too large in .mat file (columns)'
      endif
c
c  read variable name
      read(ILU2,err=998,end=999) (name(i),i=1,namlen-1),zerobyte
c
c  read real part of variable data
      DO 10 J=1,ncols
        DO 20 I=1,mrows
          read(ILU2,err=998,end=999)  rpart
          IF (I.LE.N.AND.J.LE.M) A(I,J) = rpart
   20   CONTINUE
   10 CONTINUE

      IF(IOFLAG.EQ.-1) THEN
        close(ILU2)
      ENDIF
c
c check if dimension problem
      if (mrows.GT.N.OR.ncols.GT.M) then
        rdflg = -2
        return
      endif
c
c set read flag to ok and return
      rdflg = 0
      return
c
c error during read
  998 rdflg = -1
      return
c
c  end of file
  999 rdflg = 1

      end
